%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functional Generalized Structured Component Analysis (PMET-957R2)
% Written  in MATLAB R2014a
% Last modified: April 15, 2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is for downloading and preprocessing the gait data. 

%% USER SPECIFICATION NEEDED 
% - filepath: the directory where your data file will be downloded 

%% download files from Physionet 
clear all; clc;

% set the path on which you download the data
filepath = 'D:\RESEARCH\Projects\FGSCA\data\';
mkdir(filepath);

% download demographic information file
filename_demo = strcat(filepath,'demographics.xls');
urlwrite('http://www.physionet.org/physiobank/database/gaitpdb/demographics.xls',filename_demo);
[V,text] = xlsread(filename_demo);

% extract relevant variables and Parkinson's patients only
indpd_t = find(strcmp(text(:,3),'PD')); 
indpd_v = indpd_t-1; 

demodata = V(indpd_v,[4,5,6,7,8,9,10]);
indpd_v_n = indpd_v(sum(isnan(demodata),2)==0);
indpd_t_n = indpd_v_n + 1;

% listwise deletion of missing data
demodata = demodata(sum(isnan(demodata),2)==0,:);
% column1: Height
% column2: Weight
% column3: HY
% column4: UPDRS
% column5: UPDRS_motor
% column6: TUAG (Timed Up and Go test)
% column7: Speed

% list of the names of the files to download
filename = text(indpd_t_n,1);

% download data files
for k = 1:length(filename)
    disp(strcat('downloading data file--- ',filename{k},'_01.txt'));
    urlwrite(strcat('http://www.physionet.org/physiobank/database/gaitpdb/',filename{k},'_01.txt'),strcat(filepath,filename{k},'_01.txt'));
end


%% preprocess gait data
COEF = [];
for i = 1:length(filename)
    disp(strcat('preprocessing data file--- ',strcat(filepath,filename{i},'_01.txt')));

    X = load(strcat(filepath,filename{i},'_01.txt'));
    x = X(:,18); % total force under left foot
    
    % find peak points
    x1 = x(1:end-1);
    x2 = x(2:end);
    xdiff = x2-x1;
    [pks0,loc0] = findpeaks(xdiff,'minpeakheight',40,'minpeakdistance',50);
    
    loc = [];
    for j = 1:length(loc0)
        if loc0(j)>20
            indtemp1 = [loc0(j)-20:loc0(j)];
            xtemp = x(indtemp1);
            indtemp2 = find(xtemp<20);
            loc = [loc indtemp1(max(indtemp2))];
        end
    end
    
    
    loc1 = loc(1:end-1);
    loc2 = loc(2:end);
    locdiff = loc2-loc1;
    
    % extract each gait cycle
    Datax = {};
    k = 1;
    for j = 1:length(locdiff)
        if locdiff(j)>80 & locdiff(j)<150
            temp = x(loc(j):loc(j)+locdiff(j)-1);
            tempind = find(temp<20);
            if ~isempty(tempind(tempind>50))
                Datax{k} = temp(1:min(tempind(tempind>50)));
                k = k + 1;
            end
            
        end
    end
    
    % smoothing
    range = [0,100];
    breaks = linspace(0,100,12);
    basisobj = create_bspline_basis(range,[],4,breaks);
    
    plambda = 10.^[-5:1:5];
    gcvsave=[];
    for p = 1:length(plambda)
        lambda = plambda(p);
        
        gcvsum = 0;
        for j = 1:length(Datax)
            timepoint = linspace(0,100,length(Datax{j}));
            fdparobj = fdPar(basisobj,2,lambda);
            [fdobjx,df,gcv] = smooth_basis(timepoint,Datax{j},fdparobj);
            gcvsum = gcvsum + gcv;
        end
        gcvsave(p) = gcvsum;
    end
    lambdai(i) = plambda(gcvsave==min(gcvsave));
    fdparobj = fdPar(basisobj,2,lambdai(i));
    
    COEFx = [];
    for j = 1:length(Datax)
        timepoint = linspace(0,100,length(Datax{j}));
        fdobjx = smooth_basis(timepoint,Datax{j},fdparobj);
        COEFx(:,j) = getcoef(fdobjx);
    end
    
    fdobj_ind = fd(COEFx,basisobj);
    COEF_mean(:,i) = getcoef(mean(fdobj_ind));
   
end

fdobj_gait_mean=fd(COEF_mean,basisobj);

plot(fdobj_gait_mean)
save(strcat(filepath,'gait.mat'),'fdobj_gait_mean','lambdai','demodata');

